@@ -1,6 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 import calendar
 import os
 import time
@@ -213,6 +210,11 @@ class TestDateTimeParserParse:
             == self.expected
         )
 
+    def test_parse_am(self):
+
+        with pytest.raises(ParserMatchError):
+            self.parser.parse("2021-01-30 14:00:00 AM", "YYYY-MM-DD HH:mm:ss A")
+
     def test_parse_year_two_digit(self):
 
         self.expected = datetime(1979, 1, 1, 12, 5, 10)
@@ -223,44 +225,21 @@ class TestDateTimeParserParse:
     def test_parse_timestamp(self):
 
         tz_utc = tz.tzutc()
-        int_timestamp = int(time.time())
-        self.expected = datetime.fromtimestamp(int_timestamp, tz=tz_utc)
-        assert self.parser.parse("{:d}".format(int_timestamp), "X") == self.expected
-
         float_timestamp = time.time()
+        int_timestamp = int(float_timestamp)
+        self.expected = datetime.fromtimestamp(int_timestamp, tz=tz_utc)
+        assert self.parser.parse(f"{int_timestamp:d}", "X") == self.expected
+
         self.expected = datetime.fromtimestamp(float_timestamp, tz=tz_utc)
-        assert self.parser.parse("{:f}".format(float_timestamp), "X") == self.expected
+        assert self.parser.parse(f"{float_timestamp:f}", "X") == self.expected
 
         # test handling of ns timestamp (arrow will round to 6 digits regardless)
         self.expected = datetime.fromtimestamp(float_timestamp, tz=tz_utc)
-        assert (
-            self.parser.parse("{:f}123".format(float_timestamp), "X") == self.expected
-        )
+        assert self.parser.parse(f"{float_timestamp:f}123", "X") == self.expected
 
         # test ps timestamp (arrow will round to 6 digits regardless)
         self.expected = datetime.fromtimestamp(float_timestamp, tz=tz_utc)
-        assert (
-            self.parser.parse("{:f}123456".format(float_timestamp), "X")
-            == self.expected
-        )
-
-        # NOTE: negative timestamps cannot be handled by datetime on Window
-        # Must use timedelta to handle them. ref: https://stackoverflow.com/questions/36179914
-        if os.name != "nt":
-            # regression test for issue #662
-            negative_int_timestamp = -int_timestamp
-            self.expected = datetime.fromtimestamp(negative_int_timestamp, tz=tz_utc)
-            assert (
-                self.parser.parse("{:d}".format(negative_int_timestamp), "X")
-                == self.expected
-            )
-
-            negative_float_timestamp = -float_timestamp
-            self.expected = datetime.fromtimestamp(negative_float_timestamp, tz=tz_utc)
-            assert (
-                self.parser.parse("{:f}".format(negative_float_timestamp), "X")
-                == self.expected
-            )
+        assert self.parser.parse(f"{float_timestamp:f}123456", "X") == self.expected
 
         # NOTE: timestamps cannot be parsed from natural language strings (by removing the ^...$) because it will
         # break cases like "15 Jul 2000" and a format list (see issue #447)
@@ -276,6 +255,24 @@ class TestDateTimeParserParse:
         with pytest.raises(ParserError):
             self.parser.parse(".1565982019", "X")
 
+    # NOTE: negative timestamps cannot be handled by datetime on Windows
+    # Must use timedelta to handle them: https://stackoverflow.com/questions/36179914
+    @pytest.mark.skipif(
+        os.name == "nt", reason="negative timestamps are not supported on Windows"
+    )
+    def test_parse_negative_timestamp(self):
+        # regression test for issue #662
+        tz_utc = tz.tzutc()
+        float_timestamp = time.time()
+        int_timestamp = int(float_timestamp)
+        negative_int_timestamp = -int_timestamp
+        self.expected = datetime.fromtimestamp(negative_int_timestamp, tz=tz_utc)
+        assert self.parser.parse(f"{negative_int_timestamp:d}", "X") == self.expected
+
+        negative_float_timestamp = -float_timestamp
+        self.expected = datetime.fromtimestamp(negative_float_timestamp, tz=tz_utc)
+        assert self.parser.parse(f"{negative_float_timestamp:f}", "X") == self.expected
+
     def test_parse_expanded_timestamp(self):
         # test expanded timestamps that include milliseconds
         # and microseconds as multiples rather than decimals
@@ -283,26 +280,26 @@ class TestDateTimeParserParse:
 
         tz_utc = tz.tzutc()
         timestamp = 1569982581.413132
-        timestamp_milli = int(round(timestamp * 1000))
-        timestamp_micro = int(round(timestamp * 1000000))
+        timestamp_milli = round(timestamp * 1000)
+        timestamp_micro = round(timestamp * 1_000_000)
 
         # "x" token should parse integer timestamps below MAX_TIMESTAMP normally
         self.expected = datetime.fromtimestamp(int(timestamp), tz=tz_utc)
-        assert self.parser.parse("{:d}".format(int(timestamp)), "x") == self.expected
+        assert self.parser.parse(f"{int(timestamp):d}", "x") == self.expected
 
         self.expected = datetime.fromtimestamp(round(timestamp, 3), tz=tz_utc)
-        assert self.parser.parse("{:d}".format(timestamp_milli), "x") == self.expected
+        assert self.parser.parse(f"{timestamp_milli:d}", "x") == self.expected
 
         self.expected = datetime.fromtimestamp(timestamp, tz=tz_utc)
-        assert self.parser.parse("{:d}".format(timestamp_micro), "x") == self.expected
+        assert self.parser.parse(f"{timestamp_micro:d}", "x") == self.expected
 
         # anything above max µs timestamp should fail
         with pytest.raises(ValueError):
-            self.parser.parse("{:d}".format(int(MAX_TIMESTAMP_US) + 1), "x")
+            self.parser.parse(f"{int(MAX_TIMESTAMP_US) + 1:d}", "x")
 
         # floats are not allowed with the "x" token
         with pytest.raises(ParserMatchError):
-            self.parser.parse("{:f}".format(timestamp), "x")
+            self.parser.parse(f"{timestamp:f}", "x")
 
     def test_parse_names(self):
 
@@ -345,7 +342,7 @@ class TestDateTimeParserParse:
 
         self.expected = datetime(2013, 1, 1, tzinfo=tz.gettz(full_tz_name))
         assert (
-            self.parser.parse("2013-01-01 {}".format(full_tz_name), "YYYY-MM-DD ZZZ")
+            self.parser.parse(f"2013-01-01 {full_tz_name}", "YYYY-MM-DD ZZZ")
             == self.expected
         )
 
@@ -869,7 +866,7 @@ class TestDateTimeParserRegex:
         assert escape_regex.findall("2018-03-09 8 [h] 40 [hello]") == ["[h]", "[hello]"]
 
     def test_month_names(self):
-        p = parser.DateTimeParser("en_us")
+        p = parser.DateTimeParser("en-us")
 
         text = "_".join(calendar.month_name[1:])
 
@@ -878,7 +875,7 @@ class TestDateTimeParserRegex:
         assert result == calendar.month_name[1:]
 
     def test_month_abbreviations(self):
-        p = parser.DateTimeParser("en_us")
+        p = parser.DateTimeParser("en-us")
 
         text = "_".join(calendar.month_abbr[1:])
 
@@ -939,9 +936,7 @@ class TestDateTimeParserRegex:
 
         for sep in time_seperators:
             assert time_re.findall("12") == [("12", "", "", "", "")]
-            assert time_re.findall("12{sep}35".format(sep=sep)) == [
-                ("12", "35", "", "", "")
-            ]
+            assert time_re.findall(f"12{sep}35") == [("12", "35", "", "", "")]
             assert time_re.findall("12{sep}35{sep}46".format(sep=sep)) == [
                 ("12", "35", "46", "", "")
             ]
@@ -1328,7 +1323,7 @@ class TestDateTimeParserISO:
         # ordinal in basic format: YYYYDDDD
         assert self.parser.parse_iso("1998136") == datetime(1998, 5, 16)
 
-        # timezone requires +- seperator
+        # timezone requires +- separator
         with pytest.raises(ParserError):
             self.parser.parse_iso("20180517T1055130700")
 
@@ -1445,12 +1440,12 @@ class TestDateTimeParserMonthName:
         )
 
     def test_localized_month_name(self):
-        parser_ = parser.DateTimeParser("fr_fr")
+        parser_ = parser.DateTimeParser("fr-fr")
 
         assert parser_.parse("2013-Janvier-01", "YYYY-MMMM-DD") == datetime(2013, 1, 1)
 
     def test_localized_month_abbreviation(self):
-        parser_ = parser.DateTimeParser("it_it")
+        parser_ = parser.DateTimeParser("it-it")
 
         assert parser_.parse("2013-Gen-01", "YYYY-MMM-DD") == datetime(2013, 1, 1)
 
@@ -1476,7 +1471,7 @@ class TestDateTimeParserMeridians:
         )
 
     def test_localized_meridians_lowercase(self):
-        parser_ = parser.DateTimeParser("hu_hu")
+        parser_ = parser.DateTimeParser("hu-hu")
         assert parser_.parse("2013-01-01 5 de", "YYYY-MM-DD h a") == datetime(
             2013, 1, 1, 5
         )
@@ -1486,7 +1481,7 @@ class TestDateTimeParserMeridians:
         )
 
     def test_localized_meridians_capitalized(self):
-        parser_ = parser.DateTimeParser("hu_hu")
+        parser_ = parser.DateTimeParser("hu-hu")
         assert parser_.parse("2013-01-01 5 DE", "YYYY-MM-DD h A") == datetime(
             2013, 1, 1, 5
         )
@@ -1519,7 +1514,7 @@ class TestDateTimeParserMeridians:
 @pytest.mark.usefixtures("dt_parser")
 class TestDateTimeParserMonthOrdinalDay:
     def test_english(self):
-        parser_ = parser.DateTimeParser("en_us")
+        parser_ = parser.DateTimeParser("en-us")
 
         assert parser_.parse("January 1st, 2013", "MMMM Do, YYYY") == datetime(
             2013, 1, 1
@@ -1556,19 +1551,19 @@ class TestDateTimeParserMonthOrdinalDay:
             parser_.parse("January 11st, 2013", "MMMM Do, YYYY")
 
     def test_italian(self):
-        parser_ = parser.DateTimeParser("it_it")
+        parser_ = parser.DateTimeParser("it-it")
 
         assert parser_.parse("Gennaio 1º, 2013", "MMMM Do, YYYY") == datetime(
             2013, 1, 1
         )
 
     def test_spanish(self):
-        parser_ = parser.DateTimeParser("es_es")
+        parser_ = parser.DateTimeParser("es-es")
 
         assert parser_.parse("Enero 1º, 2013", "MMMM Do, YYYY") == datetime(2013, 1, 1)
 
     def test_french(self):
-        parser_ = parser.DateTimeParser("fr_fr")
+        parser_ = parser.DateTimeParser("fr-fr")
 
         assert parser_.parse("Janvier 1er, 2013", "MMMM Do, YYYY") == datetime(
             2013, 1, 1
@@ -1608,7 +1603,7 @@ class TestDateTimeParserSearchDate:
         )
 
     def test_parse_search_locale_with_names(self):
-        p = parser.DateTimeParser("sv_se")
+        p = parser.DateTimeParser("sv-se")
 
         assert p.parse("Jan föddes den 31 Dec 1980", "DD MMM YYYY") == datetime(
             1980, 12, 31
@@ -1655,3 +1650,22 @@ class TestDateTimeParserSearchDate:
         assert self.parser.parse(
             "Dec 31, 2017 |^${}().*+?<>-& 2:00 AM", format
         ) == datetime(2017, 12, 31, 2, 0)
+
+
+@pytest.mark.usefixtures("dt_parser")
+class TestFuzzInput:
+    # Regression test for issue #860
+    def test_no_match_group(self):
+        fmt_str = str(b"[|\x1f\xb9\x03\x00\x00\x00\x00:-yI:][\x01yI:yI:I")
+        payload = str(b"")
+
+        with pytest.raises(parser.ParserMatchError):
+            self.parser.parse(payload, fmt_str)
+
+    # Regression test for issue #854
+    def test_regex_module_error(self):
+        fmt_str = str(b"struct n[X+,N-M)MMXdMM]<")
+        payload = str(b"")
+
+        with pytest.raises(parser.ParserMatchError):
+            self.parser.parse(payload, fmt_str)
